// ******************************************
// ** USB Mouse for the ZX81 & Other 8bits **
// **                                      **
// ** zx81keyboardadventure.com 2018       **
// ******************************************
// ** Copyright FREE for all to use        **
// ** If you like it use and improve it,   **
// ** share it and post it.                **
// ******************************************
// ** Version Date: 28/05/2018             **
// ******************************************

#include <hidboot.h>
#include <usbhub.h>
#include "ZeaMouse.h"

// Satisfy the IDE, which needs to see the include statement in the ino too.
#ifdef dobogusinclude
#include <spi4teensy3.h>
#include <SPI.h>
#endif

// Mouse defaults - Change to varry responsiveness
const byte gcbDebounce = 5;
const byte gcbDebounceLimit = 75;
const float gcfAcceleration = 0.1;

// **********************
// *** USB Host Setup ***
// **********************

USB     Usb;
USBHub	Hub(&Usb);
HIDBoot	<USB_HID_PROTOCOL_MOUSE>    HidMouse(&Usb);
uint32_t next_time;

// **********************************
// *** Global Variables & Classes ***
// **********************************
MouseRptParser gcMouseAction;

const byte gbJoyPinsOut[5] = {6,A1,7,8,A0}; // Up, Down, Left, Right, Fire
float gfMouseDebounce[4] = {gcbDebounce,gcbDebounce,gcbDebounce,gcbDebounce};

void setup()
{
	Serial.begin( 115200 );
	#if !defined(__MIPSEL__)
	while (!Serial); // Wait for serial port to connect - used on Leonardo, Teensy and other boards with built-in USB CDC serial connection
	#endif
	Serial.println("Start");

	if (Usb.Init() == -1)
		Serial.println("OSC did not start.");

	delay( 200 );

	next_time = millis() + 5000;

	HidMouse.SetReportParser(0, &gcMouseAction);

	byte bCount;

	for(bCount = 0 ; bCount < 5; bCount ++){
		pinMode(gbJoyPinsOut[bCount], OUTPUT);
		digitalWrite(gbJoyPinsOut[bCount], LOW);
	}

	pinMode(LED_BUILTIN,OUTPUT);
	digitalWrite(LED_BUILTIN, LOW);
}

void fn_AxisOff (byte bJoyPin){

	// Mouse Decceleration
	if (gfMouseDebounce[bJoyPin] > gcbDebounce - gcfAcceleration*1.5){
		gfMouseDebounce[bJoyPin] -= gcfAcceleration*1.5;
	} else {
		gfMouseDebounce[bJoyPin] = gcbDebounce;
	}
	digitalWrite(gbJoyPinsOut[bJoyPin], LOW);
}

void loop()
{
	byte bCount;
	Usb.Task();
	float fMouseCheck = 0;


	// Read XY movements of mouse
	for(bCount = 0 ; bCount < 4; bCount ++){

		fMouseCheck = millis() - gcMouseAction.plMovedLastMills[bCount];

		if(fMouseCheck < gfMouseDebounce[bCount]){

			// Mouse Acceleration
			if (gfMouseDebounce[bCount] < gcbDebounceLimit ) {
				// Higher than 22 freaks out the ZX81
				gfMouseDebounce[bCount]+=gcfAcceleration;

				// Turn of Oposite movement
				fn_AxisOff(bCount+-1+(((bCount+1) % 2)*2));

				// Turn on Movement
				digitalWrite(gbJoyPinsOut[bCount], HIGH);
			}
		} else if(fMouseCheck > gfMouseDebounce[bCount]) {
			fn_AxisOff(bCount);
		}

	}

	// Read Button Press. Mouse button has higher priority in this case
	// as the mouse button reports pressed until released.
	if(gcMouseAction.plMovedLastMills[4]){
		digitalWrite(gbJoyPinsOut[4], HIGH);
	} else {
		digitalWrite(gbJoyPinsOut[4], LOW);
	}
}







